% Generate a synthetic ECG signal
Fs = 1000; % Sampling frequency (Hz)
t = 0:1/Fs:10; % Time vector for a 10-second signal
% Generate ECG components with realistic P, QRS, and T waves
P_wave = 0.1 * cos(2 * pi * 1.2 * t - pi/2); % P wave
QRS_freq = 2.2 + 0.1 * sin(2 * pi * 0.2 * t); % Varying frequency for QRS complex
Q_wave = 0.005 * cos(2 * pi * QRS_freq .* t + pi); % Q wave
R_wave = 0.5 * cos(2 * pi * QRS_freq .* t); % R wave
S_wave = 0.05 * cos(2 * pi * 1.3 * t); % S wave
T_wave = 0.1 * cos(2 * pi * 1.8 * t - pi/4); % T wave
% Construct the ECG signal with noise
ecg_signal = P_wave + Q_wave - R_wave - S_wave + T_wave;
noise = 0.2 * randn(size(t));
noisy_ecg_signal = ecg_signal + noise;
% Plot the original and noisy ECG signals
figure;
subplot(2,1,1);
plot(t, ecg_signal);
title('Original ECG Signal');
subplot(2,1,2);
plot(t, noisy_ecg_signal, 'r');
title('Noisy ECG Signal');
% Denoising using a bandpass filter
lowcut = 0.5;
highcut = 50;
filtered_ecg_signal = bandpass_filter(noisy_ecg_signal, lowcut, highcut, Fs);
% Plot the original, noisy, and filtered ECG signals
figure;
subplot(3,1,1);
plot(t, ecg_signal);
title('Original ECG Signal');
subplot(3,1,2);
plot(t, noisy_ecg_signal, 'r');
title('Noisy ECG Signal');
subplot(3,1,3);
plot(t, filtered_ecg_signal, 'g');
title('Filtered ECG Signal');
% R-peak detection using Pan-Tompkins algorithm
[~,locs] = findpeaks(filtered_ecg_signal, 'MinPeakHeight', 0.5, 'MinPeakDistance', Fs/2);
% Plot the filtered ECG signal with detected R-peaks
figure;
plot(t, filtered_ecg_signal, 'g');
hold on;
plot(t(locs), filtered_ecg_signal(locs), 'bx', 'MarkerSize', 10);
title('Filtered ECG Signal with R-peak Detection');
